import requests
from dfk_commons.classes.Token import Token


class APIService:
    def __init__(self, url, api_key, chain):
        self.url = url
        self.headers = {
            'x-api-key': api_key,
            'Content-Type': 'application/json'
        }
        self.tokens = self.getTokens(chain)
        self.contracts = self.getContracts(chain)
        self.pairs = self.getPairs("Jewel", chain)

    def getTokens(self, chain):
        response = requests.get(f"{self.url}/dfk/tokens", headers=self.headers,  params={"chain": chain})
        response_json = response.json()
       
        if response.status_code != 200: raise Exception(f"API Error: {response_json}")
        tokens = {}
        for token in response_json:
            tokens[token] = Token(token, chain, response_json[token]["address"], response_json[token]["decimals"], None)
        return tokens

    def getPairs(self, base_token, chain):
        response = requests.get(f"{self.url}/dfk/pairs", headers=self.headers, params={"token": base_token, "chain": chain})
        response_json = response.json()
        if response.status_code != 200: raise Exception(f"API Error: {response_json}")
        return response_json
    
    def getContracts(self, chain):
        response = requests.get(f"{self.url}/dfk/contracts", headers=self.headers, params={"chain": chain})
        response_json = response.json()
        if response.status_code != 200: raise Exception(f"API Error: {response_json}")
        return response_json
    


class RPCProvider:
     def __init__(self, chain, provider, url, chainId) -> None:
            self.w3 = provider
            self.chain = chain
            self.url = url
            self.chainId = chainId
         






import boto3
from boto3 import Session

class TablesManager:
    def __init__(self, prod: bool, dup: bool) -> None:
        self.session: Session = boto3.session.Session(
            region_name = "us-east-1",
        )
        self.prod: bool = prod
        self.dup = dup

        #autoplayer tables
        self.accounts = None
        self.autoplayer = None
        self.managers = None

        #gas tables
        self.gas = None
        self.mining_gas = None
        self.gardening_gas = None

        #history tables
        self.history = None
        self.payouts = None
        self.fees = None

        #tracking tables
        self.buyer_tracking = None
        self.autoplayer_tracking = None
        self.profit_tracker = None

        #trading tables
        self.trades = None
        self.active_orders = None
        self.trade_errors = None

        #dup user tables
        self.settings = None
        self.users = None
        self.user_filters = None

        if dup:
            self.init_dup_tables()
        else:
            self.init_default_tables()

    def init_default_tables(self):
        if self.prod:
            self.accounts = self.session.resource('dynamodb').Table("dfk-autoplayer-accounts")
            self.autoplayer = self.session.resource('dynamodb').Table("dfk-autoplayer")

            #trading tables
            self.trades = self.session.resource('dynamodb').Table("dfk-trading-trades")
            self.active_orders = self.session.resource('dynamodb').Table("dfk-trading-active-orders")
            self.trade_errors = self.session.resource('dynamodb').Table("dfk-trading-errors")
        else:
            self.accounts = self.session.resource('dynamodb').Table("dfk-autoplayer-accounts-dev")
            self.autoplayer = self.session.resource('dynamodb').Table("dfk-autoplayer-dev")

            #trading tables
            self.trades = self.session.resource('dynamodb').Table("dfk-trading-trades-dev")
            self.active_orders = self.session.resource('dynamodb').Table("dfk-trading-active-orders-dev")
            self.trade_errors = self.session.resource('dynamodb').Table("dfk-trading-errors-dev")

        self.managers = self.session.resource('dynamodb').Table("dfk-autoplayer-managers")

        #gas tables
        self.gas = self.session.resource('dynamodb').Table("dfk-autoplayer-gas")
        self.mining_gas = self.session.resource('dynamodb').Table("dfk-autoplayer-mining-gas")
        self.gardening_gas = self.session.resource('dynamodb').Table("dfk-autoplayer-gardening-gas")

        #history tables
        self.history = self.session.resource('dynamodb').Table("dfk-autoplayer-history")
        self.payouts = self.session.resource('dynamodb').Table("dfk-autoplayer-payouts")
        self.fees = self.session.resource('dynamodb').Table("dfk-autoplayer-fee")

        #tracking tables
        self.buyer_tracking = self.session.resource('dynamodb').Table("dfk-buyer-tracking")
        self.autoplayer_tracking = self.session.resource('dynamodb').Table("dfk-autoplayer-tracking")
        self.profit_tracker = self.session.resource('dynamodb').Table("dfk-profit-tracker")
    
    def init_dup_tables(self):
        self.accounts = self.session.resource('dynamodb').Table("accounts")
        self.settings = self.session.resource('dynamodb').Table("settings")
        self.users = self.session.resource('dynamodb').Table("users")
        self.user_filters= self.session.resource('dynamodb').Table("user-filters")
    


from dfk_commons.abi_getters import RouterABI
from dfk_commons.classes.APIService import APIService
from dfk_commons.classes.Account import Account
from dfk_commons.classes.RPCProvider import RPCProvider
from dfk_commons.classes.Token import Token
import time


def buyToken(account: Account, amount: int, expected_cost: int, token: Token, apiService: APIService, rpcProvider: RPCProvider):
    RouterContract = rpcProvider.w3.eth.contract(address=apiService.contracts["Router"]["address"], abi=RouterABI)
    tx = RouterContract.functions.swapETHForExactTokens(
        amount,
        [apiService.tokens["Jewel"].address, token.address],
        account.address,
        int(time.time()+60)
        
    ).build_transaction({
        "from": account.address,
        "nonce": account.nonce,
        "value": expected_cost
    })
    tx["gas"] = int(rpcProvider.w3.eth.estimate_gas(tx))
    tx["maxFeePerGas"] = rpcProvider.w3.to_wei(50, 'gwei')
    tx["maxPriorityFeePerGas"] = rpcProvider.w3.to_wei(3, "gwei")
    signed_tx = rpcProvider.w3.eth.account.sign_transaction(tx, account.key)
    hash = rpcProvider.w3.eth.send_raw_transaction(signed_tx.rawTransaction)
    hash = rpcProvider.w3.to_hex(hash)
    rpcProvider.w3.eth.wait_for_transaction_receipt(hash)



from dfk_commons.abi_getters import ERC721ABI
from dfk_commons.classes.APIService import APIService
from dfk_commons.classes.Account import Account
from dfk_commons.classes.RPCProvider import RPCProvider


def sendHero(account: Account, receiver: str, heroId: int, apiService: APIService, rpcProvider: RPCProvider):
    tx =  rpcProvider.w3.eth.contract(address=apiService.contracts["Heroes"]["address"], abi=ERC721ABI).functions.safeTransferFrom(account.address, receiver, int(heroId)).build_transaction({
        "from": account.address,
        "nonce": account.nonce,
    })
    signed_tx =  rpcProvider.w3.eth.account.sign_transaction(tx, account.key)
    hash =  rpcProvider.w3.eth.send_raw_transaction(signed_tx.rawTransaction)
    hash =  rpcProvider.w3.to_hex(hash)